/****************************************************************
* Modul:   TA.BA_MMT.HS1301 - Multimedia Technik (MMT)
* Subject: Processing - Uebung 4b Midi
* Author:  Felix Rohrer
* Desc:    Midi Player
****************************************************************/
import java.io.*;
import java.util.*;
import java.util.concurrent.TimeUnit;
import javax.sound.midi.*;
import ddf.minim.*;
import ddf.minim.analysis.*;


// ----- init Vars ----------------------------------------------
int canvasWidth = 450;
int canvasHeight = 220;
int currentSongIndex = 0;

File folder;
List<Sequence> songList = new ArrayList<Sequence>();
List<String> songName = new ArrayList<String>();
Sequencer sequencer;
String currentSong;
boolean isPlaying = false;

String currTime, totalTime, displayText;
long currMillis, totalMillis;

AudioInput in;
Minim minim;
FFT fft;

// ----- setup() ------------------------------------------------
void setup() {
  size(canvasWidth, canvasHeight);
  frameRate(24);
  background(45);
  textSize(16);
  
  try {
    sequencer = MidiSystem.getSequencer();
    sequencer.open();
  }
  catch (Exception e)
  {
    System.out.println("no sequencer...");
    e.printStackTrace();
    System.exit(-1);
  }
  
  if (sequencer == null)
  {
     // Fehlerbehandlung
     System.out.println("no sequencer...");
     System.exit(-1);
  }
  System.out.println("sequencer is ready...");

  
  // get all files from data folder, create playlist
  folder = new File(dataPath(""));
  for (String file : folder.list()) {
    if (file.toLowerCase().endsWith(".mid")) {
      File myMidiFile = new File(dataPath("") + "/" + file);
      try {
        //System.out.println("File: " + myMidiFile);
        songName.add(file.replace("_", " ").replace(".mid", ""));
        songList.add(MidiSystem.getSequence(myMidiFile));
     }
      catch (Exception e)
      {
        System.out.println("songList.add()...");
        e.printStackTrace();
        System.exit(-1);
      }
    }
  }
  System.out.println("songList size(): " + songList.size());

  
  try {
    sequencer.setSequence(songList.get(currentSongIndex));
    currentSong = songName.get(currentSongIndex);
  }
  catch (Exception e)
      {
        System.out.println("setSequence()...");
        e.printStackTrace();
        System.exit(-1);
      }
      
      
  // init Minim & FFT
  minim = new Minim(this);
  in = minim.getLineIn(Minim.MONO, 16384, 44100);
  fft = new FFT(in.bufferSize(), in.sampleRate());
  fft.window(FFT.HAMMING);
  
  
  sequencer.start();
  System.out.println("start()...");
  isPlaying = true;
}

// ----- draw() -------------------------------------------------
void draw() {
  
  background(45);
  
  // Play Soundlist
  playSound();
  
  // draw waveform
  stroke(255);
  saturation(10);
  for(int i = 0; i < in.bufferSize()-1; i++)
  {
    line(i,   80 + in.mix.get(i)*50,
         i+1, 80 + in.mix.get(i+1)*50);
  }
  
  
  // draw FFT
  stroke(175);
  fft.forward(in.mix);
  for (int i = 0; i < fft.specSize(); i++) {
    line(i, height, i, height - fft.getBand(i) * 4);
  }
  
  // fetch current play position
  float playPos = sequencer.getTickPosition();
  float playLen = sequencer.getTickLength();
  float xpos = (playPos / playLen) * width;
  
  // draw progress bar
  fill(alpha(color(0,126,255,102)));
  stroke(alpha(color(0,126,255,102)));
  rect (0, 0, xpos, 20);

  // show song title
  fill(255);
  
  currMillis = (long) playPos;
  totalMillis = (long) playLen;
  
  currTime  = String.format("%d:%d", TimeUnit.MILLISECONDS.toMinutes(currMillis),
                                      TimeUnit.MILLISECONDS.toSeconds(currMillis)
                                      - TimeUnit.MINUTES.toSeconds(TimeUnit.MILLISECONDS.toMinutes(currMillis)));
  totalTime = String.format("%d:%d", TimeUnit.MILLISECONDS.toMinutes(totalMillis),
                                      TimeUnit.MILLISECONDS.toSeconds(totalMillis)
                                      - TimeUnit.MINUTES.toSeconds(TimeUnit.MILLISECONDS.toMinutes(totalMillis)));
  
  //text (currentSong, 2, 0, width, 200);
    text (currentSong + " (" + currTime + " /" + totalTime + ")", 2, 0, width, 200);
}

// ----- playSound() --------------------------------------------
void playSound() {
  if (isPlaying && !sequencer.isRunning()) {
    playNextSound();
  }
}

// ----- playNextSound() ----------------------------------------
void playNextSound() {
  if (currentSongIndex >= songList.size() - 1) {
    currentSongIndex = -1;
  }
  sequencer.stop();
    
  currentSongIndex++;
  try
  {
    sequencer.setSequence(songList.get(currentSongIndex));
    currentSong = songName.get(currentSongIndex);
  }
  catch (Exception e)
      {
        System.out.println("setSequence() #2...");
        e.printStackTrace();
        System.exit(-1);
      }
  // init new FFT
  //fft = null;
  //fft = new FFT(currentSong.bufferSize(), currentSong.sampleRate());
  //fft.window(FFT.HAMMING);
  
  sequencer.start();
}

// ----- keyPressed() -------------------------------------------
void keyPressed() {
  if (key == 'n') {
    playNextSound();
  }
  else if (key == ' ') {
    if (sequencer.isRunning()) {
      sequencer.stop();
      isPlaying = false;
    } else {
      sequencer.start();
      isPlaying = true;
    }
  }
}

// ----- mousePressed() -----------------------------------------
void mousePressed() {
  // get click coordinates
  float pos = ((float) mouseX / width) * sequencer.getTickLength();
  // jump to position
  sequencer.setTickPosition((int) pos);
}


// ----- stop() -------------------------------------------------
void stop()
{
  sequencer.stop();
  //minim.stop();
  super.stop();
}

